untyped
global function glitchSkyboxPrecache
global function glitchSkyboxSettings

void function glitchSkyboxSettings(){
    #if UI
        #if S2_PLAYER_HAS_MOD_SETTINGS_GLITCH_SKY
            AddModTitle("S2.GlitchOverhaul Skybox")
            AddModCategory("Skybox Basic")
            AddConVarSetting("s2_glitch_skybox_set_colour", "Colour", "vector")
            AddConVarSettingEnum("s2_glitch_skybox_rgb", "RGB Cycle", ["Off", "On :D"])
            AddConVarSetting("s2_glitch_skybox_alpha", "Alpha", "float")
            AddConVarSetting("s2_glitch_skybox_rate", "Scroll Rate", "float")
            AddConVarSetting("s2_glitch_skybox_angle", "Scroll Angle", "float")

            AddModCategory("Skybox Advanced")
            AddConVarSettingEnum("s2_glitch_skybox_advanced", "Enabled", ["Off", "On"])

            AddConVarSetting("s2_glitch_skybox_adv_r_period", "Red Sine Period", "float")
            AddConVarSetting("s2_glitch_skybox_adv_r_offset", "Red Sine Offset", "float")
            AddConVarSetting("s2_glitch_skybox_adv_r_min", "Red Sine Min", "float")
            AddConVarSetting("s2_glitch_skybox_adv_r_max", "Red Sine Max", "float")

            AddConVarSetting("s2_glitch_skybox_adv_g_period", "Green Sine Period", "float")
            AddConVarSetting("s2_glitch_skybox_adv_g_offset", "Green Sine Offset", "float")
            AddConVarSetting("s2_glitch_skybox_adv_g_min", "Green Sine Min", "float")
            AddConVarSetting("s2_glitch_skybox_adv_g_max", "Green Sine Max", "float")

            AddConVarSetting("s2_glitch_skybox_adv_b_period", "Blue Sine Period", "float")
            AddConVarSetting("s2_glitch_skybox_adv_b_offset", "Blue Sine Offset", "float")
            AddConVarSetting("s2_glitch_skybox_adv_b_min", "Blue Sine Min", "float")
            AddConVarSetting("s2_glitch_skybox_adv_b_max", "Blue Sine Max", "float")
        #endif
    #endif
}

void function glitchSkyboxPrecache(){
    #if CLIENT
    thread glitchSkyboxMain()
    #endif
}

#if CLIENT

vector function GetConVarFloat3(string convar){
    array<string> value = split(GetConVarString(convar), " ")
    try{
        return Vector(value[0].tofloat(), value[1].tofloat(), value[2].tofloat()) 
    }
    catch(ex){
        throw "Invalid convar " + convar + "! make sure it is a float3 and formatted as \"X Y Z\""
    }
    unreachable
}

float function sine_k(float aMax, float aMin){
    return ((aMax + aMin)/2)
}

float function sine_s(float aMax, float aMin){
    return fabs((aMax - aMin)/2)
}

void function glitchSkyboxMain(){
    for(;;){
        WaitFrame()
        if(GetMapName() != "mp_glitch")
            continue
        bool usesAdvanced = (GetConVarInt("s2_glitch_skybox_advanced") == 1)
        bool usesRainbow = (GetConVarInt("s2_glitch_skybox_rgb") == 1)
        if(!usesAdvanced){
            if(usesRainbow){
                SetConVarFloat("s2_glitch_skybox_vmt_r", sin(Time()))
                SetConVarFloat("s2_glitch_skybox_vmt_g", sin(Time() - 2*PI/3))
                SetConVarFloat("s2_glitch_skybox_vmt_b", sin(Time() - 4*PI/3))
                continue
            }
            vector colourSet = GetConVarFloat3("s2_glitch_skybox_set_colour")
            SetConVarFloat("s2_glitch_skybox_vmt_r", colourSet.x)
            SetConVarFloat("s2_glitch_skybox_vmt_g", colourSet.y)
            SetConVarFloat("s2_glitch_skybox_vmt_b", colourSet.z)
            continue
        }
        float redPeriod = GetConVarFloat("s2_glitch_skybox_adv_r_period")
        float redOffset = (GetConVarFloat("s2_glitch_skybox_adv_r_offset")/3)
        float redMin = GetConVarFloat("s2_glitch_skybox_adv_r_min")
        float redMax = GetConVarFloat("s2_glitch_skybox_adv_r_max")

        float greenPeriod = GetConVarFloat("s2_glitch_skybox_adv_g_period")
        float greenOffset = (GetConVarFloat("s2_glitch_skybox_adv_g_offset")/3)
        float greenMin = GetConVarFloat("s2_glitch_skybox_adv_g_min")
        float greenMax = GetConVarFloat("s2_glitch_skybox_adv_g_max")

        float bluePeriod = GetConVarFloat("s2_glitch_skybox_adv_b_period")
        float blueOffset = (GetConVarFloat("s2_glitch_skybox_adv_b_offset")/3)
        float blueMin = GetConVarFloat("s2_glitch_skybox_adv_b_min")
        float blueMax = GetConVarFloat("s2_glitch_skybox_adv_b_max")

        float advColR = (sine_s(redMax, redMin)*sin(((Time()*(1/redPeriod)) - redOffset)*2*PI)) + sine_k(redMax, redMin)
        float advColG = (sine_s(redMax, greenMin)*sin(((Time()*(1/greenPeriod)) - greenOffset)*2*PI)) + sine_k(redMax, greenMin)
        float advColB = (sine_s(blueMax, blueMin)*sin(((Time()*(1/bluePeriod)) - blueOffset)*2*PI)) + sine_k(blueMax, blueMin)

        SetConVarFloat("s2_glitch_skybox_vmt_r", advColR)
        SetConVarFloat("s2_glitch_skybox_vmt_g", advColG)
        SetConVarFloat("s2_glitch_skybox_vmt_b", advColB)
    }
}

#endif